#ifndef NODES_H
#define NODES_H

#include <stdio.h>
#include <stdlib.h>

typedef struct Program {
    struct Statement * statement;
} Program;

typedef enum {
    WHILE_STATEMENT,
    PRINT_STATEMENT,
    FOR_STATEMENT,
    IF_STATEMENT,
    INTEGRATE_STATEMENT,
    DERIVATIVE_STATEMENT,
    EVALUATE_STATEMENT,
    ASSIGN_VARIABLE_STATEMENT,
    DECLARE_VARIABLE_STATEMENT,
    DECLARE_FUNCTION_STATEMENT
} StatementType;

typedef struct Statement {
    StatementType type;

    struct While * while_st;
    struct Print * print;
    struct For * for_st;
    struct If * if_st;
    struct Integrate * integrate;
    struct Derivative * derivative;
    struct Evaluate * evaluate;
    struct AssignVariable * assignVariable;
    struct Variable * declareVariable;
    struct Function * declareFunction;
    
    struct Statement * next;
} Statement;

typedef struct Variable {
    char * name;
    struct Expression * value;
} Variable;

typedef struct While {
    struct Boolean * boolean;
    struct Block * block;
} While;

typedef enum {
    DERIVATIVE_EXPRESSION,
    INTEGRATE_EXPRESSION,
    EVALUATE_EXPRESSION,
    NEGATIVE_EXPRESSION,
    PARENTHESIS_EXPRESSION,
    NUMBER_EXPRESSION,
    ADD_EXPRESSION,
    SUB_EXPRESSION,
    MUL_EXPRESSION,
    DIV_EXPRESSION,
    POW_EXPRESSION,
    ID_EXPRESSION
} ExpressionType;

typedef struct Expression {
    ExpressionType type;

    struct Expression * leftExpression;
    struct Expression * rightExpression;
    // struct Variable * variable;
    struct Derivative * derivative;
    struct Integrate * integrate;
    struct Evaluate * evaluate;
    struct NegativeExp * negative;
    struct ParenthesisExp * parenthesis;
    double value;
    char * id;

    struct Expression * next;
} Expression;

typedef enum {
    EXPRESSION_FUNCTION,
    EXPRESSION_DOMAIN_FUNCTION,
    COMPOSITE_FUNCTION,
} FunctionType;

typedef struct Function {
    FunctionType type;

    char * name;

    struct Expression * expression;
    struct Domain * domain;
    struct Composite * composite;

    struct Function * next;
} Function;

typedef struct Block {
    Statement * statement;
} Block;

typedef enum {
    EXPRESSION_PRINT,
    STRING_PRINT
} PrintType;

typedef struct Print {
    PrintType type;

    Expression * expression;
    char * string;
} Print;

typedef struct For {
    Variable * init_variable;
    struct AssignVariable * action_variable;
    struct Boolean * boolean;
    Block * block;
} For;

typedef struct If {
    struct Boolean * boolean;
    Block * block;
    Block * else_block;
} If;

typedef enum {
    DOMAIN_BOOLEAN,
    OR_BOOLEAN,
    AND_BOOLEAN,
    BOOLEAN,
    NOT_BOOLEAN
} BooleanType;

typedef struct Boolean {
    BooleanType type;

    struct Domain * domain;
    struct Boolean * left;
    struct Boolean * right;
} Boolean;

typedef enum {
    GREATER_DOMAIN,
    GREATER_EQUAL_DOMAIN,
    LESSER_EQUAL_DOMAIN,
    LESSER_DOMAIN,
    EQUAL_DOMAIN,
    NOT_EQUAL_DOMAIN,
    LESSER_LESSER_DOMAIN,
    LESSER_LESSER_EQUAL_DOMAIN,
    LESSER_EQUAL_LESSER_EQUAL_DOMAIN,
    LESSER_EQUAL_LESSER_DOMAIN
} DomainType;

typedef struct Domain {
    DomainType type;

    Expression * left;
    Expression * mid;
    Expression * right;
} Domain;

typedef struct Derivative {
    char * function;
    Expression * value;
    int * grade;
    double * error;
} Derivative;

typedef struct Evaluate {
    char * function;
    Expression * value;
} Evaluate;

typedef struct Integrate {
    char * function;
    double * error;

    Expression * lvalue;
    Expression * rvalue;
} Integrate;

typedef enum {
    ID_COMPOSITE,
    COMPOSITE_COMPOSITE
} CompositeType;

typedef struct Composite {
    CompositeType type;
    char * outter_func;

    struct Composite * inner_comp;
    char * inner_func;
} Composite;

// typedef struct NumArray {
//     Expression * expressions;
//     size_t size;

//     struct NumArray * next;
// } NumArray;

typedef enum {
    ADD_ASSIGN_EXPRESSION,
    SUB_ASSIGN_EXPRESSION,
    MUL_ASSIGN_EXPRESSION,
    DIV_ASSIGN_EXPRESSION,
    INCREMENT_EXPRESSION,
    DECREMENT_EXPRESSION,
    ASSIGN_EXPRESSION
} AssignVariableType;

typedef struct AssignVariable {
    AssignVariableType type;
    char * id;
    Expression * expression;
} AssignVariable;

#endif