%{

#include "bison-actions.h"
#include "../../backend/support/nodes.h"

%}

%union {
    Program * program;
    Variable * variable;
    If * if_st;
    For * for_st;
    While * while_st;
    Print * print;
    Function * function;
    Statement * statement;
    Expression * expression;
    Derivative * derivative;
    Evaluate * evaluate;
    Integrate * integrate;
    Boolean * boolean;
    Domain * domain;
    Block * block;
    Composite * composite;
    // NumArray * numArray;
    AssignVariable * assignVariable;

    int intNumber;
    double doubleNumber;
    char * string;
    int token;
}

%type <program> program
%type <variable> declare_variable
%type <assignVariable> assign_variable
%type <function> function functions declare_function
%type <statement> statement statements
%type <expression> expression
%type <derivative> derivative
%type <integrate> integrate
%type <evaluate> evaluate
%type <boolean> boolean
%type <domain> domain
%type <block> block
%type <while_st> while_statement
%type <if_st> if_statement
%type <for_st> for_statement
%type <composite> composite
%type <print> print
/* %type <numArray> numArrays numArray */
%type <doubleNumber> positive_constant
%type <string> escapeString
/* %type <intNumber> positive_constant */

%token <token> ADD
%token <token> SUB
%token <token> MUL
%token <token> DIV
%token <token> POW
%token <token> FUNCTION
%token <token> IF
%token <token> ELSE
%token <token> FOR
%token <token> WHILE
%token <token> PRINT
%token <token> SEMICOLON
%token <token> COMPOSITE
%token <token> DERIVATIVE
%token <token> INTEGRATE
%token <token> BETWEEN
%token <token> AND_INT
%token <token> EVALUATE
%token <token> IN
%token <token> OPEN_BRACES
%token <token> OPEN_BRACKETS
%token <token> CLOSE_BRACES
%token <token> CLOSE_BRACKETS
%token <token> OPEN_PARENTHESIS
%token <token> CLOSE_PARENTHESIS
%token <token> ASSIGN
%token <token> EQUAL
%token <token> AND
%token <token> OR
%token <token> NOT
%token <token> ADD_ASSIGN
%token <token> INCREMENT
%token <token> SUB_ASSIGN
%token <token> DECREMENT
%token <token> MUL_ASSIGN
%token <token> DIV_ASSIGN
%token <string> DQUOTE
%token <token> COMMA
%token <token> COLON
%token <token> LESSER
%token <token> LESSER_EQUAL
%token <token> GREATER
%token <token> GREATER_EQUAL
%token <token> NOT_EQUAL
%token <token> VAR
%token <intNumber> ERR
%token <string> ESCAPESTRING
%token <doubleNumber> DOUBLE
%token <intNumber> INTEGER
%token <string> ID

%precedence CLOSE_PARENTHESIS
%precedence ELSE

%left GREATER GREATER_EQUAL LESSER LESSER_EQUAL EQUAL NOT_EQUAL
%left ADD SUB
%right NOT
%left MUL DIV AND OR
%left POW

%%

program: statements                                                                 { $$ = ProgramGrammarAction($1); }
    ;

statement: evaluate SEMICOLON                                                       { $$ = StatementEvaluateGrammarAction($1); }
    | integrate SEMICOLON                                                           { $$ = StatementIntegrateGrammarAction($1); }
    | derivative SEMICOLON                                                          { $$ = StatementDerivativeGrammarAction($1); }
    | if_statement                                                                  { $$ = StatementIfGrammarAction($1); }
    | declare_function SEMICOLON                                                    { $$ = StatementDeclareFunctionGrammarAction($1); }
    | for_statement                                                                 { $$ = StatementForGrammarAction($1); }
    | while_statement                                                               { $$ = StatementWhileGrammarAction($1); }
    | print SEMICOLON                                                               { $$ = StatementPrintGrammarAction($1); }
    | declare_variable SEMICOLON                                                    { $$ = StatementDeclareVariableGrammarAction($1); }
    | assign_variable SEMICOLON                                                     { $$ = StatementAssignVariableGrammarAction($1); }
    ;

expression: positive_constant                                                       { $$ = ExpressionPositiveConstantGrammarAction($1); }
    | evaluate                                                                      { $$ = ExpressionEvaluateGrammarAction($1); }
    | integrate                                                                     { $$ = ExpressionIntegrateGrammarAction($1); }
    | derivative                                                                    { $$ = ExpressionDerivativeGrammarAction($1); }
    | ID                                                                            { $$ = ExpressionIdGrammarAction($1); }
    | expression ADD expression                                                     { $$ = ExpressionOperatorAddGrammarAction($1, $3); }
    | expression SUB expression                                                     { $$ = ExpressionOperatorSubGrammarAction($1, $3); }
    | expression MUL expression                                                     { $$ = ExpressionOperatorMulGrammarAction($1, $3); }
    | expression DIV expression                                                     { $$ = ExpressionOperatorDivGrammarAction($1, $3); }
    | expression POW expression                                                     { $$ = ExpressionOperatorPowGrammarAction($1, $3); }
    | OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                                 { $$ = ExpressionParenthesisGrammarAction($2); }
    | SUB expression                                                                { $$ = ExpressionNegativeGrammarAction($2); }
    ;

domain: expression LESSER expression                                                { $$ = DomainLesserOperatorGrammarAction($1, $3); }
    | expression LESSER_EQUAL expression                                            { $$ = DomainLesserEqualOperatorGrammarAction($1, $3); }
    | expression GREATER_EQUAL expression                                           { $$ = DomainGreaterEqualOperatorGrammarAction($1, $3); }
    | expression GREATER expression                                                 { $$ = DomainGreaterOperatorGrammarAction($1, $3); }
    | expression EQUAL expression                                                   { $$ = DomainEqualOperatorGrammarAction($1, $3); }
    | expression NOT_EQUAL expression                                               { $$ = DomainNotEqualOperatorGrammarAction($1, $3); }
    | expression LESSER expression LESSER expression                                { $$ = DomainLesserLesserOperatorGrammarAction($1, $3, $5); }
    | expression LESSER_EQUAL expression LESSER expression                          { $$ = DomainLesserEqualLesserOperatorGrammarAction($1, $3, $5); }
    | expression LESSER expression LESSER_EQUAL expression                          { $$ = DomainLesserLesserEqualOperatorGrammarAction($1, $3, $5); }
    | expression LESSER_EQUAL expression LESSER_EQUAL expression                    { $$ = DomainLesserEqualLesserEqualOperatorGrammarAction($1, $3, $5); }
    ;

boolean: domain                                                                     { $$ = BooleanDomainGrammarAction($1); }
    | boolean OR boolean                                                            { $$ = BooleanOrGrammarAction($1, $3); }
    | boolean AND boolean                                                           { $$ = BooleanAndGrammarAction($1, $3); }
    | OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS                                    { $$ = BooleanGrammarAction($2); }
    | NOT boolean                                                                   { $$ = NotBooleanAction($2); }
    ;

positive_constant: DOUBLE                                                           { $$ = $1; }
    | INTEGER                                                                       { $$ = (double) $1; }
    ;

declare_function: FUNCTION ID ASSIGN function                                       { $$ = DeclareFunctionGrammarAction($2, $4); }
    | FUNCTION ID ASSIGN OPEN_BRACES functions CLOSE_BRACES                         { $$ = DeclareFunctionGrammarAction($2, $5); }
    ;

function: expression                                                                { $$ = ExpressionFunctionGrammarAction($1); }
    | expression COLON domain                                                       { $$ = ExpressionDomainFunctionGrammarAction($1, $3); }
    | composite                                                                     { $$ = CompositeFunctionGrammarAction($1); }
    ;

functions: function                                                                 { $$ = $1; }
    | function COMMA functions                                                      { $$ = FunctionsGrammarAction($1, $3); }
    ;

composite: ID COMPOSITE ID                                                          { $$ = CompositeIdGrammarAction($1, $3); }
    | ID COMPOSITE composite                                                        { $$ = CompositeCompGrammarAction($1, $3); }
    ;

/* expressions: expression                                                             { $$ = $1; }
    | expression COMMA expressions                                                  { $$ = ExpressionsGrammarAction($1, $3); }
    ; */

/* ids: ID                                                                             { $$ = $1 }
    | ID COMMA ids                                                                  { $$ = DoubleGrammarAction($1) }
    ; */

/* numArray: OPEN_BRACKETS expressions CLOSE_BRACKETS                                  { $$ = $2; }
    ;

numArrays: numArray                                                                 { $$ = DoubleGrammarAction($1) }
    | numArray COMMA numArrays                                                      { $$ = DoubleGrammarAction($1) }
    ;

arrayNumArray: OPEN_BRACKETS numArrays CLOSE_BRACKETS                               { $$ = DoubleGrammarAction($1) }
    ; */

/* idArray: OPEN_BRACKETS ids CLOSE_BRACKETS                                           { $$ = DoubleGrammarAction($1) }
    ; */

escapeString: ESCAPESTRING                                                          { $$ = EscapedStringGrammarAction($1); }
    | ESCAPESTRING ADD escapeString                                                 { $$ = EscapedStringsGrammarAction($1, $3); }
    ;
 /*
escapeStrings: escapeString                                                         { $$ = $1 }
    | escapeString COMMA escapeStrings                                              { $$ = DoubleGrammarAction($1) }
    ;

escapeStringArray: OPEN_BRACKETS escapeStrings CLOSE_BRACKETS                       { $$ = DoubleGrammarAction($1) }
    ;
*/
evaluate: EVALUATE ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS              { $$ = EvaluateGrammarAction($2, $5); }
    /* | EVALUATE ID IN numArray                                                       { $$ = DoubleGrammarAction($1) } */
    | ID OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                              { $$ = EvaluateGrammarAction($1, $3); }
    /* | EVALUATE idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS             { $$ = DoubleGrammarAction($1) } */
    /* | EVALUATE idArray IN numArray                                                  { $$ = DoubleGrammarAction($1) } */
    /* | idArray OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                         { $$ = DoubleGrammarAction($1) } */
    ;

integrate: INTEGRATE ID BETWEEN OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                     { $$ = IntegrateGrammarAction($2, $5, $7); }
    | INTEGRATE ID OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                                  { $$ = IntegrateGrammarAction($2, $4, $6); }
    /* | INTEGRATE idArray BETWEEN OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                     { $$ = DoubleGrammarAction($1) }
    | INTEGRATE idArray OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS                             { $$ = DoubleGrammarAction($1) } */
    | integrate ERR positive_constant                                                                                { $$ = ErroredIntegrateGrammarAction($1, $3); }
    ;

print: PRINT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS                          { $$ = PrintExpressionGrammarAction($3); }
    | PRINT OPEN_PARENTHESIS escapeString CLOSE_PARENTHESIS                         { $$ = PrintStringGrammarAction($3); }
    ;

statements: statement                                                               { $$ = $1; }
    | statement statements                                                          { $$ = StatementsGrammarAction($1, $2); }
    ;

derivative: DERIVATIVE ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS          { $$ = SimpleDerivativeGrammarAction($2, $5); }
    | DERIVATIVE INTEGER ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS        { $$ = GradedDerivativeGrammarAction($2, $3, $6); }
    /* | DERIVATIVE idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS           { $$ = DoubleGrammarAction($1) }
    | DERIVATIVE INTEGER idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS   { $$ = DoubleGrammarAction($1) } */
    | derivative ERR positive_constant                                              { $$ = ErroredDerivativeGrammarAction($1, $3); }
    ; 

block: OPEN_BRACES statements CLOSE_BRACES                                          { $$ = BlockGrammarAction($2); }
    | statement                                                                     { $$ = BlockGrammarAction($1); }
    ;

if_statement: IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block                   { $$ = IfBlockGrammarAction($3, $5); }
    | IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block ELSE block                { $$ = IfElseBlockGrammarAction($3, $5, $7); }
    ;

for_statement: FOR OPEN_PARENTHESIS declare_variable SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS block        { $$ = ForBlockGrammarAction($3, $5, $7, $9); }
    | FOR OPEN_PARENTHESIS SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS block                                  { $$ = ForBlockNoVarGrammarAction($4, $6, $8); }
    ;

while_statement: WHILE OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block             { $$ = WhileBlockGrammarAction($3, $5); }
    ;

declare_variable: VAR ID                                                            { $$ = DeclareVariableGrammarAction($2); }
    | VAR ID ASSIGN expression                                                      { $$ = DeclareValuedVariableGrammarAction($2, $4); }
    ;

assign_variable: ID ADD_ASSIGN expression                                           { $$ = AssignVariableAddGrammarAction($1, $3); }
    | ID SUB_ASSIGN expression                                                      { $$ = AssignVariableSubGrammarAction($1, $3); }
    | ID MUL_ASSIGN expression                                                      { $$ = AssignVariableMulGrammarAction($1, $3); }
    | ID DIV_ASSIGN expression                                                      { $$ = AssignVariableDivGrammarAction($1, $3); }
    | ID INCREMENT                                                                  { $$ = AssignVariableIncGrammarAction($1); }
    | ID DECREMENT                                                                  { $$ = AssignVariableDecGrammarAction($1); }
    | ID ASSIGN expression                                                          { $$ = AssignVariableGrammarAction($1, $3); }
    ;

%%
