#include "backend/code-generation/generator.h"
#include "backend/semantic-analysis/analyzer.h"
#include "backend/free-tree/free-tree.h"
#include "backend/support/logger.h"
#include "backend/support/shared.h"
#include "frontend/syntactic-analysis/bison-parser.h"
#include <stdio.h>
#include <math.h>
#include <getopt.h>

CompilerState state;

int main(int argc, char ** argv) {
    // #ifdef YYDEBUG
    //  	yydebug = 1;
    // #endif

    state.succeed = false;

    char * outputFile = NULL;
    if (argc > 1) {
        FILE * file = fopen(argv[1], "r+");
        if (file == NULL) {
            perror("Error abriendo el archivo");
            abort();
        }
        yyin = file;
        
        for (int i = 1; i < argc; ++i) {
            argv[i] = argv[i+1];
        }
        argc--;

        int c = getopt(argc, argv, "ho:");
        switch (c) {
            case 'h':
                LogInfo("Suckerberg");
                exit(EXIT_SUCCESS);
            case 'o':
                outputFile = optarg;
                break;
            default:
                break;
        }
    }

    InitializeMallocTable();
    
    int result = yyparse();
    switch (result) {
        case 0:
            if (state.succeed) {
                if (!Analyze(state.program)) {
                    result = EXIT_FAILURE;
                    break;
                }

                FILE * file = fopen(outputFile != NULL ? outputFile : "bottler.m", "w+");
                if (file == NULL) {
                    perror("Error abriendo el archivo");
                    result = 1;
                    break;
                }

                Generator(state.program, file);

                if (fclose(file) == EOF) {
                    perror("Error cerrando el archivo");
                    result = EXIT_FAILURE;
                    break;
                }
            }
            else {
                LogError("Se produjo un error en la aplicacion.");
                result = EXIT_FAILURE;
                break;
                // return -1;
            }
            break;
        case 1:
            LogError("Bison finalizo debido a un error de sintaxis.");
            break;
        case 2:
            LogError("Bison finalizo abruptamente debido a que ya no hay memoria disponible.");
            break;
        default:
            LogError("Error desconocido mientras se ejecutaba el analizador Bison (codigo %d).", result);
    }

    FreeMallocTable();
    return result;
}
